'(
Program do jednocipu AVR ATmega_48 pro ovladani 6-ti relatkoveho prepinace
anten. Verze s moznosti uzivatelskeho nastaveni sepnutych rele. Reaguje na BCD kod
(band dada z Elecraft K3, Yaesu FT-1000 apod.), nebo na napeti na vstupu
AD prevodniku ADC(5). Napeti pro jednotliva pasma odpovidaji standardu YAESU,
napr. FT817. (napeti 0.33V, 0,67V, 1,0V, 1,33V, atd...)
')

Rem ***** deklarace promennych, konfigurace portu, AD prevodniku atd... ****************************

$regfile = M48def.dat                   'odkaz na soubor M48def.dat pro kompilaci
$crystal = 8000000                      'kmitocet oscilatoru 8MHz. (neni krystal ale vnitrni RC oscilator)

Dim Band(13) As Eram Byte               'pole s tabulkou sepnutych rele podle BCD vstupu(+1) jako indexu pole (v EEprom)
Dim Band_2(13) As Eram Byte             'pole s tabulkou sepnutych rele podle napeti na vstupu ADC (tabulka v EEprom)
Dim Rele(7) As Byte                     'pole ocislovanych sepnutych rele
Rele(1) = &B00000000
Rele(2) = &B00000001                    'nastaveni hodnot ocislovanych sepnutych rele
Rele(3) = &B00000010                    'v binarnim tvaru je dobre videt, co je sepnuto
Rele(4) = &B00000100
Rele(5) = &B00001000
Rele(6) = &B00010000
Rele(7) = &B00100000
Dim Bcd_vst As Byte                     'promenna BCD vstupu z radia
Dim X As Byte                           'pomocna promenna pri nastavovani a ukladani hodnot do EEprom (index pole Band(13))
Dim Y As Byte                           'pomocna promenna pro rucni ovladani prepinace (index pole Rele(7))
Y = 2                                   'pocatecni hodnota indexu pro rucni ovladani (Rele(2) = 1. rele )
Dim Vstup As Byte                       'cislo pasma na vstupu pro ADC (pasmo dane napetim na ADC vstupu)
Dim Volty As Word                       'hodnota nactena ADC (nejde o napeti ale o cislo odpovidajici napeti na vstupu)

'Konfigurace IN / OUT portu
Config Portb.0 = Output                 'Piny B.0 az B.5 jsou vystupy na rele
Config Portb.1 = Output
Config Portb.2 = Output
Config Portb.3 = Output
Config Portb.4 = Output
Config Portb.5 = Output
Reset Portb                             'resetovany na 0 (odpojena vsechna rele)

Config Portd = Input                    'cely port D (8 pinu) jako vstupni
Set Portd.0                             'piny D.0 az D.3 jsou vstupy BCD kodu z radia
Set Portd.1                             'vsem se nastavi (interne) pull-up resistory
Set Portd.2
Set Portd.3
Set Portd.4                             'pin D.4 je tlacitko proti zemi  (interne nastaven pull-up resistor)
Set Portd.5                             'pin D.5 je tlacitko proti zemi (interne nastaven pull-up resistor)
Config Portd.6 = Output                 'pin D.6 je vystupni; zluta LED proti zemi (s odporem) indikuje rucni ovladani
Config Portd.7 = Output                 'pin D.7 je vystupni; cervena LED proti zemi (s odporem) indikuje rezim nastavovani
Reset Portd.6                           'LED na pinu  D.6 (po restartu nesviti)
Reset Portd.7                           'LED na pinu D.7 (po restartu nesviti)

Config Portc.2 = Input                  'uroven na pinu C.2 urcuje podle ceho se prepinaji pasma
Set Portc.2                             'Log 1 (rozpojeny kontakt) = vstup BCD; Log 0 (zkrat na zem) = vstup napeti

Config Debounce = 30                    'casove zpozdeni pro test stisku tlacitka (osetreni zakmitu kontaktu) 30ms

'konfigurace AD prevodniku
Config Adc = Single , Prescaler = Auto  'nastaveni parametru ADC prevodniku

Rem ******** konec deklaraci a konfiguraci, zacina vlastni program *****************************



Rem **** odskok na nastaveni default pri podrzeni tlacitka na Pinu D.5 pri zapnuti napajeni***
Debounce Pind.5 , 0 , Default , Sub     'podrzet tlacitko Pin_D.5 pri startu  = odskok na default nastaveni
Bitwait Pind.5 , Set                    'ceka na uvolneni tlacitka
Reset Portd.6                           'zhasne obe LED
Reset Portd.7
Rem ***  konec default nastaveni ****


Rem ******* hlavni smycka programu prepinani podle BCD na vstupu  *******
Decoder_bcd:
Do

Bcd_vst = Pind And &B00001111           'nacteni hodnoty BCD kodu z radia (+ maska 4 hornich bitu)
Bcd_vst = Bcd_vst + 1                   'zvyseni o 1; (BCD jde od 0 ale indexy pole Band(13) jdou od 1)
Portb = Band(bcd_vst)                   'nastavi na portu B odpovidajici vystupy na rele podle hodnot ulozenych v poli Band(13) v EEprom

If Pind.4 = 1 Then                      '...NEstisknute tlacitko na pinu D.4 a zaroven...
   Debounce Pind.5 , 0 , Rucne , Sub    '... stisknute tlacitko na pinu D.5 = odskok na rucni ovladani
End If

If Pind.4 = 0 Then                      'Podrzet tlacitko na Pin_D.4 ...
   Debounce Pind.5 , 0 , Nastaveni , Sub       '... a zaroven kratce stisknout tlacitko na Pin_D.5 = odskok na nastaveni
End If

If Pinc.2 = 0 Then                      'pokud je zkratovana propojka na pinu C.2 na zem, tak ...
    Exit Do                             '... vyskoc ze smycky a ....
End If

Loop
Rem *** konec hlavni smycky programu prepinani podle BCD na vstupu  ***

Goto Komparator                         '...skoc do casti pro prepinani podle napeti.




Komparator:
Rem ****** hlavni smycka prepinani podle napeti na vstupu *******

Do
    Gosub Napeti                        'odskok do podprogramu zmereni napeti a prirazeni pasma
    Waitms 30                           'pauza 30ms
    Portb = Band_2(vstup)               'nastavi na portu B odpovidajici vystupy na rele podle hodnot ulozenych v poli Band_2(13) v EEprom

If Pind.4 = 1 Then                      '...NEstisknute tlacitko na pinu D.4 a zaroven...
   Debounce Pind.5 , 0 , Rucne , Sub    '... stisknute tlacitko na pinu D.5 = odskok na rucni ovladani
End If

If Pind.4 = 0 Then                      'Podrzet tlacitko na Pin_D.4 ...
   Debounce Pind.5 , 0 , Nastaveni_2 , Sub       '... a zaroven kratce stisknout tlacitko na Pin_D.5 = odskok na nastaveni
End If

If Pinc.2 = 1 Then                      'pokud je rozpojena propojka na pinu C.2 ...
    Exit Do                             '...  tak vyskocit ze smycky a ...
End If

Loop                                    'konec smycky prepinani podle napeti

Goto Decoder_bcd                        ' ... skocit na  dekodovani ze vstupu BCD

Rem *************** konec hlavni smycky prepinani podle napeti na vstupu *******************


Napeti:
Rem ************* mereni napeti ****************************************************************
Rem **** sem se odskakuje z hlavni smycky prepinani podle napeti na vstupu *************

 Start Adc                              'zapnut? A/D p?evodn?ku
     Volty = Getadc(5)                  'zji?t?n? hodnoty z p?evodn?ku ADC(5) a ulo?en? do prom?nn? "Volty"
 Stop Adc                               'vypnut? A/D p?evodn?ku

If Volty > 41 And Volty < 94 Then       'pokud je zmerena hodnota mezi 41 a 94 tak...
 Vstup = 2                            '... nejake pasmo na vstupu
      Elseif Volty > 110 And Volty < 164 Then       'pokud je zmerena hodnota mezi 110 a 164 tak...
 Vstup = 3                            '... jine pasmo na vstupu, ...
      Elseif Volty > 178 And Volty < 231 Then       'atd ...
 Vstup = 4
      Elseif Volty > 246 And Volty < 299 Then
    Vstup = 5
      Elseif Volty > 315 And Volty < 368 Then
    Vstup = 6
      Elseif Volty > 383 And Volty < 436 Then
    Vstup = 7
      Elseif Volty > 450 And Volty < 503 Then
    Vstup = 8
      Elseif Volty > 520 And Volty < 573 Then
    Vstup = 9
      Elseif Volty > 587 And Volty < 640 Then
    Vstup = 10
      Elseif Volty > 655 And Volty < 708 Then
    Vstup = 11
      Elseif Volty > 724 And Volty < 777 Then
    Vstup = 12
      Elseif Volty > 792 And Volty < 845 Then
    Vstup = 13
Else                                    ' pokud zmerena hodnota nevyhovuje zadne podmince vyse,tak ...
    Vstup = 1                           '... vstup 1 (vsechna rele jsou rozpojena)
End If
Return                                  'navrat do hlavni smycky prepinani podle napeti na vstupu
'************************************************************************************************


Nastaveni:
Rem ****** Nastaveni  predvolby pro automaticke prepinani anten podle BCD na vstupu *********************
Rem ****** Nastavi a ulozi se pro kazde pasmo (kazdy BCD kod na vstupu) individualne ********************

Portb = 255                             'probliknuti vsemi LED (rele)
Waitms 500                              'po dobu 500ms
Portb = 0                               'odpojeni vsech rele
Waitms 200                              'pauza 200ms
Set Portd.7                             'rozsviceni LED na pinu D.7 (doporucuji cervenou, indikuje rezim nastavovani)
Bitwait Pind.5 , Set                    'cekani na uvolneni tlacitka na Pinu D.5

Bcd_vst = Pind And &B00001111           'nacteni hodnoty BCD kodu z radia (+ maska 4 hornich bitu)
Bcd_vst = Bcd_vst + 1                   'zvyseni o 1; (BCD jde od 0 ale indexy promenne Band(13) jdou od 1)
Portb = Band(bcd_vst)                   'nastaveni portu B (vystupy na rele) podle hodnoty ulozene v tabulce v EEprom Band(13)
X = Bcd_vst                             'zkopirovani hodnoty BCD kodu do promenne X

'zacatek smycky nastaveni
Do

'krokovani rele tlacitkem na pinu D.4
If Pind.4 = 0 Then                      'Pri stisku tlacitka na Pinu D.4 ...
   Incr X                               '...zvysit hodnotu X o jednicku
      If X > 7 Then                     '...pokud by X bylo vetsi nez 7 ...
      X = 1                             '... nastavit ho zase na 1 ...
      End If
   Portb = Rele(x)                      'Nastaveni ant.rele podle hodnoty z pole Rele(X) odpovidajici indexu X
   Waitms 200                           'pauza 200ms
End If

'ulozeni a navrat
If Pind.5 = 0 Then                      'Pri stisku tlacitka na Pinu D.5
   Band(bcd_vst) = Rele(x)              'Ulozeni hodnoty z pole Rele(X) do tabulky Band(13) v EEprom
   Waitms 200                           'Pauza 200ms
   Portb = 255                          'probliknuti vsech LED (rele)
   Waitms 500                           ' ... 500ms ...
   Portb = 0                            '... zhasnuti vsech LED (rele)
   Exit Do                              'vyskoceni ze smycky
 End If
'konec smycky nastaveni
 Loop

Reset Portd.7                           'zhasnuti LED na pinu D.7 (doporucuji cervenou)
Return                                  'navrat do hlavni smycky

Rem ************ konec nastaveni  predvoleb podle BCD na vstupu ******************

Nastaveni_2:
Rem ******** Nastaveni  predvoleb pro automaticke prepinani anten podle napeti ****************
Rem ******** Nastavi a ulozi se pro kazde pasmo (vstupni napeti) individualne *****************

Portb = 255                             'probliknuti vsemi LED (rele)
Waitms 500                              'po dobu 500ms
Portb = 0                               'odpojeni vsech rele
Waitms 200                              'pauza 200ms
Set Portd.7                             'rozsviceni LED na pinu D.7 (doporucuji cervenou, indikuje rezim nastavovani)
Bitwait Pind.5 , Set                    'cekani na uvolneni tlacitka na Pinu D.5

Gosub Napeti                            'odskok do podprogramu zmereni napeti a prirazeni pasma
Portb = Band_2(vstup)                   'nastaveni portu B (vystup na rele) podle hodnoty ulozene v tabulce v EEprom Band_2(13)
X = Vstup                               'zkopirovani hodnoty ziskane v podprogramu "Napeti" do promenne X

'zacatek smycky nastavovani
Do

'krokovani rele tlacitkem na pinu D.4
If Pind.4 = 0 Then                      'Pri stisku tlacitka n Pin_D.4 ...
   Incr X                               '...zvysit hodnotu X o jednicku
      If X > 7 Then                     '...pokud by X bylo vetsi nez 7 ...
      X = 1                             '... nastavit ho zase na 1 ...
      End If
   Portb = Rele(x)                      'Nastaveni ant.rele podle hodnoty z pole Rele(13) odpovidajici indexu X
   Waitms 200                           'pauza 200ms
End If

'ulozeni a navrat
If Pind.5 = 0 Then                      'Pri stisku tlacitka na Pin_D.5
   Band_2(vstup) = Rele(x)              'Ulozeni hodnoty z pole Rele(X)do tabulky Band_2(13) v EEprom
   Waitms 200                           'Pauza 200ms
   Portb = 255                          'probliknuti vsech LED (rele)
   Waitms 500                           ' ... 500ms ...
   Portb = 0                            '... zhasnuti vsech LED (rele)
   Exit Do                              'vyskoceni ze smycky
End If
'konec smycky nastaveni
 Loop

Reset Portd.7                           'zhasnuti LED na pinu D.7 (doporucuji cervenou)
Return                                  'navrat do hlavni smycky
'*********************************************************************************************************

Rucne:
Rem ********* Rucni ovladani prepinace tlacitkem na Pin_D.4  *********************************************

Set Portd.6                             'Rozsviceni LED na pinu D.6 (doporucuji zlutou, indikuje ru?ni prepinani anten)
Portb = 0                               'Rozpojeni vsech rele
Waitms 200                              'Cekani 200ms

Do
Portb = Rele(y)                         'Nastaveni ant.rele podle hodnoty z pole Rele(7) odpovidajici indexu Y
Debounce Pind.4 , 0 , Dalsi , Sub       'Pri stisku tlacitka na Pin_D.4 odskok na podprogram "Dalsi" kde se zvysi Y o jednu
   If Pind.5 = 0 Then                   'Pri stisku tlacitka na Pin_D.5 navrat na automaticke ovladani
         Portb = 0                      'Rozpojeni vsech rele
         Waitms 200                     'Cekani 200ms
         Reset Portd.6                  'Zhasnuti LED na pinu D.6 (doporucuji zlutou)
         Exit Do                        'ukonceni smycky
   End If
Loop
Return                                  'Navrat do hlavni smycky programu

Dalsi:
Rem ****** zvyseni hodnoty Y o 1 (nasledujici rele v rade) **********************************************

Incr Y
      If Y > 7 Then
         Y = 1
      End If
Waitms 30
Return
Rem ********************************************************************************************************

Default:
Rem ****************** default nastaveni *****************************
Rem ***** sem odskok pri podrzeni tlacitka D.5 pri zapnuti ***********
Rem ***** nastavi se stejne pro oba rezimy (BCD i napeti) ************

Band(1) = Rele(1)                       '0000 0000
Band(2) = Rele(2)                       '0000 0001
Band(3) = Rele(3)                       '0000 0010
Band(4) = Rele(4)                       '0000 0100
Band(5) = Rele(4)                       '0000 0100
Band(6) = Rele(5)                       '0000 1000
Band(7) = Rele(6)                       '0001 0000
Band(8) = Rele(5)                       '0000 1000
Band(9) = Rele(6)                       '0001 0000
Band(10) = Rele(5)                      '0000 1000
Band(11) = Rele(7)                      '0010 0000
Band(12) = Rele(1)                      '0000 0000
Band(13) = Rele(1)                      '0000 0000

Band_2(1) = Rele(1)                     '0000 0000
Band_2(2) = Rele(2)                     '0000 0001
Band_2(3) = Rele(3)                     '0000 0010
Band_2(4) = Rele(4)                     '0000 0100
Band_2(5) = Rele(4)                     '0000 0100
Band_2(6) = Rele(5)                     '0000 1000
Band_2(7) = Rele(6)                     '0001 0000
Band_2(8) = Rele(5)                     '0000 1000
Band_2(9) = Rele(6)                     '0001 0000
Band_2(10) = Rele(5)                    '0000 1000
Band_2(11) = Rele(7)                    '0010 0000
Band_2(12) = Rele(1)                    '0000 0000
Band_2(13) = Rele(1)                    '0000 0000

Set Portd.6                             'rozsviceni obou LED  (indikuje, ze je hotovo)
Set Portd.7                             'rozsviceni obou LED  (indikuje, ze je hotovo)

 Return                                 'navrat
Rem *******************************************************************************************************

End                                     'end program